// stdafx.h : include file for standard system include files,
// or project specific include files that are used frequently, but
// are changed infrequently
//
#pragma once
#include "targetver.h"
#include <string>
#include <iostream>
#include <math.h>
#include <stdio.h>

using namespace std;
#include <stdio.h>
#include <tchar.h>
const double NONEXIST=-65536.;
class GeoObj;
class Line;
class Point;
class Circle;
class ChildList;
enum det_type{//determine type
	FREE,
	CIR_NEAR,
	CIR_FAR,
	LIN,
	START_END,
	CENTER_RADIUS,
	CENTER
};
enum lin_type{//line type
	SEGMENT=0,
	RAY=1,
	LINE=2
};
enum geo_type{
	GLINE,
	GCIRCLE,
	GPOINT
};
struct ChildNode{//used in ChildList
	struct ChildNode * pre;
	struct ChildNode * next;
	GeoObj* value;
};

// TODO: reference additional headers your program requires here
class ChildList{//ChildList is used to record all childrens of a geo-obj
public:
	ChildList();
	struct ChildNode* start;
	struct ChildNode* end;
	int num;
	void add(GeoObj* value);
	void del(GeoObj* node);
	ChildNode* find(GeoObj* node);
	~ChildList();
};
struct Node{
	GeoObj* value;
	Node* next;
};
class List{
public:
	Node* start;
	Node* end;
	int num;
	List();
	void add(GeoObj* GeoObj);
	void del(GeoObj* Obj);
};
class Sketch{
public:
	double height;
	double width;
	List* eleList;
	Sketch();
	void paint();
};
extern Sketch* _sketch;
extern FILE* _resultfile;
class GeoObj{
public:	
	det_type determine_type;//how the object is determined
	string name;//name shown on the graph
	int color;//color shown on the graph
	int parentnum;//parentnums,0 for none, 1 for only the first, 2 for only the second, 3 for both
	bool visible;//whether it is shown on the graph
	GeoObj* parent[2];//the parents of the object
	ChildList* children;//the children of the object
	Sketch* sketch;//the sketchpad it belongs to
	geo_type type;//the type of the object: line,circle or point
	bool visited;//used to update graph
	int getcolor();
	int setcolor(int color);
	bool isvisible();
	bool setvisible(bool visible);
	void setname(string name);
	string getname();
	virtual bool update(void)=0;
	virtual void paint(void)=0;
	virtual bool hasnon(void)=0;
	void unlock();
	GeoObj(string name="",bool isvisible=true);
	static double* intercept(GeoObj* obj1,GeoObj* obj2);
	~GeoObj();
};
class Point:public GeoObj{
protected:
	double X;
	double Y;
public:
	friend class GeoObj;friend class Line;friend class Point;friend class Circle;
	Point();
	static double dist(Point* point1, Point* point2);
	static Point* intersect(GeoObj* GeoObj1,GeoObj* GeoObj2,bool isnear);
	bool hasnon(void);
	bool update(void);
	bool setXY(double X,double Y);
	double getX();
	double getY();
	void paint();
};
class Line:public GeoObj{
protected:
	double X1,Y1,X2,Y2;
public:
	friend class GeoObj;friend class Line;friend class Point;friend class Circle;
	lin_type line_type;
	Line();
	bool update(void);
	double length();
	static Line* LineXY(Point* P1,Point* P2,lin_type line_type=SEGMENT);
	double getX1();
	double getY1();
	double getX2();
	double getY2();
	Point* getStart();
	Point* getEnd();
	bool setP1(double X1,double Y1);
	bool setP2(double X2,double Y2);
	void paint();
	bool hasnon(void);
	int getLineType();
};
class Circle:public GeoObj{
protected:
	double X,Y;
	double radius;
public:
	friend class GeoObj;friend class Line;friend class Point;friend class Circle;
	Circle();
	static Circle* DrawCircle(Point* center, Line* radius);
	static Circle* DrawCircle(Point* center, double radius);
	bool update(void);
	double getX();
	double getY();
	double getRadius();
	bool setCenter(double X,double Y);
	bool setRadius(double r);
	Point* getCenter();
	void paint();
	bool hasnon(void);
};


typedef void(*func_pointer)();
enum type{
	var_num,
	var_float,
	var_string,
	var_bool,
	var_point,
	var_circle,
	var_line
};
class variable{
public:
	variable();
	type var_type;
	union {
		int* val_int;
		double* val_float;
		bool* val_bool;
		string* val_string;
		Point* val_point;
		Circle* val_circle;
		Line* val_line;
	}var_strorage;
	variable* next;
	string name;
};
class function{
public:
	func_pointer funcp;
	function* next;
	string name;
};
class Symbol_Table_Node{
public:
	Symbol_Table_Node* parent;
	variable* First;
	variable* Last;
	function* Firstf;
	function* Lastf;
	bool operator!=(const Symbol_Table_Node &orig)const;
	Symbol_Table_Node(Symbol_Table_Node* parent);
	~Symbol_Table_Node();
	Symbol_Table_Node* add_node();
	variable* add_var(int value,string name);
	variable* add_var(string value,string name);
	variable* add_var(double value,string name);
	variable* add_var(bool value,string name);
	variable* add_var(Point* value,string name);
	variable* add_var(Circle* value,string name);
	variable* add_var(Line* value,string name);
	void add_function(func_pointer funcp,string name);
	void* get_var(string name,bool is_local=false);
	func_pointer get_function(string name,bool is_local=false);
};
//*************************************
class lib{
public:
	static int print(string a);
	static double sin(double x);
	static Line* LineST(Point* P1,Point* P2,int line_type);
	static Circle* DrawCircle(Point* center, Line* radius);
	static Circle* DrawCircle(Point* center, double radius=NONEXIST);
	static Point* intersect(GeoObj* GeoObj1,GeoObj* GeoObj2,bool isnear);
	static void paint();
	Point* lib::PointXY(double X,double Y);
	Line* LineXY(double X1,double Y1,double X2,double Y2,int line_type);
	Circle* CircleXYr(double X,double Y,double r);
};